#!/usr/bin/env python3
# coding: utf-8

import joker.environ


class GlobalInterface(joker.environ.GlobalInterface):
    package_name = 'joker.superuser'

    default_config = {
        "urls": {
            "youtube.com/watch": ["v"],
            "youtube.com/playlist": ["list"],
        }
    }




#!/usr/bin/env python3
# coding: utf-8

import argparse
import shlex
import sys
from joker.superuser.environ import GlobalInterface

import pyperclip
from joker.textmanip.url import url_simplify

gi = GlobalInterface()


def smart_url_simplify(url, queries):
    config = gi.conf['urls']
    for pattern, _queries in config.items():
        if pattern in url:
            queries = _queries
            break
    return str(url_simplify(url, queries))


def _copy_and_print(url: str):
    print(url)
    print(shlex.quote(url))
    pyperclip.copy(url)


def run(prog, args):
    desc = 'simplify a url'
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    parser.add_argument('url')
    parser.add_argument('query', nargs='*')
    ns = parser.parse_args(args)
    url = str(smart_url_simplify(ns.url, ns.query))
    _copy_and_print(url)


def runloop(prog, args):
    while True:
        userinput = input('URL> ')
        parts = shlex.split(userinput)
        if not parts:
            continue
        url = parts[0]
        queries = parts[1:]
        try:
            url = smart_url_simplify(url, queries)
        except Exception as e:
            print(e.__class__.__name__, e, file=sys.stderr)
            continue
        _copy_and_print(url)



#!/usr/bin/env python3
# coding: utf-8

from __future__ import unicode_literals

import volkanic

cmddef = """
pydir       joker.superuser.tools.pydir
pyentry     joker.superuser.tools.pyentry
unsource    joker.superuser.tools.unsource
cases       joker.superuser.tools.cases
dup         joker.superuser.tools.dedup
setop       joker.superuser.tools.setop
rmdir       joker.superuser.tools.remove
apt         joker.superuser.tools.apt
url         joker.superuser.tools.urls
urls        joker.superuser.tools.urls:runloop
"""

_prog = 'python3 -m joker.superuser'
registry = volkanic.CommandRegistry.from_cmddef(cmddef, _prog)

if __name__ == '__main__':
    registry()



#!/usr/bin/env python3
# coding: utf-8

from joker.stream.shell import ShellStream


def _minus(path1, path2):
    s1 = set(ShellStream.open(path1).dense())
    s2 = set(ShellStream.open(path2).dense())
    for x in s1 - s2:
        print(x)


def run(prog, args):
    _minus(args[0], args[1])



#!/usr/bin/env python3
# coding: utf-8

import subprocess


"""
command: pmset -g batt https://stackoverflow.com/a/42361337

Now drawing from 'AC Power'
 -InternalBattery-0 (id=3735651)	97%; charged; 0:00 remaining present: true
 
Now drawing from 'Battery Power'
 -InternalBattery-0 (id=3735651)	97%; discharging; (no estimate) present: true
"""


class MacPowerStatusQuery(object):
    cmd_args = ['pmset', '-g', 'batt']

    def __init__(self):
        p = subprocess.run(self.cmd_args, capture_output=True, text=True)
        self.resp = p.stdout

    def is_using_ac(self):
        return b'AC Power' in self.resp

    def is_using_battery(self):
        return b'Battery Power' in self.resp



#!/usr/bin/env python3
# coding: utf-8
import argparse
import re
import shlex


def extract_case(path, title='subcmd'):
    text = open(path).read()
    regex = re.compile(
        r'case\s+(.*?)\s+in(.*?)\besac\b',
        re.DOTALL | re.MULTILINE,
    )
    for mat in regex.finditer(text):
        if mat and title in mat.group():
            return mat


def quotation_closed(s):
    try:
        return shlex.split(s)
    except ValueError:
        pass


def extract_pat_from_entry(entry):
    cnt = entry.count(')')
    while True:
        idx = entry.find(')')
        if idx == -1:
            break
        pat = entry[:idx]
        if not pat or pat[-1] == '\\':
            break
        if quotation_closed(pat):
            return pat


def extract_pats(text):
    for entry in text.split(';;'):
        pat = extract_pat_from_entry(entry)
        if pat:
            yield pat


def run(prog, args):
    desc = 'extract patterns of a case construct from a shell script'
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    aa = pr.add_argument
    aa('path', metavar='PATH', help='path to a shell script')
    aa('title', default='subcmd', nargs='?')
    ns = pr.parse_args(args)
    mat = extract_case(ns.path, ns.title)
    if not mat:
        return
    pats = []
    for pat in extract_pats(mat.group(2)):
        pat = ' '.join(shlex.split(pat, comments=True))
        pats.append(pat)
    if pats:
        print(ns.title + ':')
        for pat in pats:
            print('-', pat)



#!/usr/bin/env python3
# coding: utf-8

import os
import re
import shlex

import requests
from joker.cast.collective import CircularReferenceDetector
from joker.stream import Stream
from joker.stream.shell import RecursiveInclusionStream, ShellStream

regex_http = re.compile(r'https?://')


def _abspath(path):
    return os.path.abspath(os.path.expanduser(path))


def http_get(url):
    return requests.get(url).text


class UnsourceStream(RecursiveInclusionStream, ShellStream):
    def __init__(self, file, *filters):
        super(UnsourceStream, self).__init__(file, *filters)
        self.crd = None

    def include(self, locator):
        return self.openloc(locator, self.crd)

    @classmethod
    def openloc(cls, locator, crd=None):
        if isinstance(locator, str) and regex_http.match(locator):
            stm = cls.wrap(http_get(locator))
            crd_ident = locator
        else:
            stm = super(UnsourceStream, cls).open(locator, mode='r')
            crd_ident = _abspath(locator)
        if crd is None:
            stm.crd = CircularReferenceDetector(crd_ident)
        else:
            stm.crd = crd.branch(crd_ident)
        stm.name = locator
        return stm


def get_rc_path(name, userdir=True):
    if userdir:
        return _abspath('~/.' + name)
    from joker.superuser import utils
    utils.make_joker_superuser_dir()
    return utils.under_joker_superuser_dir(name)


def has_sourced(loc, t_loc):
    t_loc = t_loc if regex_http.match(t_loc) else _abspath(t_loc)
    ustm = UnsourceStream.openloc(loc)
    for line in ustm:
        s_loc = ustm.check_for_inclusion(line)
        if not s_loc:
            continue
        s_loc = s_loc if regex_http.match(s_loc) else _abspath(s_loc)
        if s_loc == t_loc:
            return True
    return False


def add_source(path, target):
    if has_sourced(path, target):
        return
    with open(path, 'a') as fout:
        line = '\nsource ' + shlex.quote(target) + '\n'
        fout.write(line)


def unsource(outpath, *locators):
    with Stream.open(outpath, 'w') as fout:
        for loc in locators:
            with UnsourceStream.openloc(loc) as ustm:
                for line in ustm.setup().dense():
                    fout.write(line + os.linesep)


def run(prog, args):
    import argparse
    desc = 'expand shell scripts with sourced files replaced by contents'
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    pr.add_argument('-a', '--apply', choices=['zshrc', 'bashrc'])
    pr.add_argument('files', nargs='+', help='shell scripts')
    ns = pr.parse_args(args)
    if not ns.apply:
        return unsource('-', *ns.files)
    outpath = get_rc_path(ns.apply, userdir=False)
    unsource(outpath, *ns.files)
    add_source(get_rc_path(ns.apply, userdir=True), outpath)



#!/usr/bin/env python3
# coding: utf-8
import subprocess


def find_name(rootdir='.', pattern=''):
    # https://unix.stackexchange.com/questions/118959/how-to-find-files-that-contain-newline-in-filename
    subprocess.run(['find', rootdir, '-name', pattern])
